﻿using Rhino;
using Rhino.Commands;
using Command = Rhino.Commands.Command;
using Rhino.Input;
using Rhino.Input.Custom;
using System;
using Eto.Forms;
using Rhino.UI;
using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace CommandWithWindow
{
    public class CommandWithWindowCommand : Command
    {
        public CommandWithWindowCommand()
        {
            Instance = this;
        }

        public static CommandWithWindowCommand Instance { get; private set; }

        public override string EnglishName => "CommandWithWindowCommand";

        protected override Result RunCommand(RhinoDoc doc, RunMode mode)
        {
            var go = new GetOption();
            go.SetCommandPrompt("Form or Dialog");
            var formIndex = go.AddOption("Form");
            var dialogIndex = go.AddOption("Dialog");
            var index = -1;
            if (go.Get() == GetResult.Option)
            {
                index = go.OptionIndex();
            }

            if (index == formIndex)
            {
                var form = new CommandForm();
                form.Show();
                RhinoApp.WriteLine("form.Show(), thread continues and command returns too early");
            }
            else if (index == dialogIndex)
            {
                var dialog = new CommandDialog();
                dialog.ShowSemiModal(doc, RhinoEtoApp.MainWindow);
                RhinoApp.WriteLine("dialog.ShowSemiModal, thread waited until dialog closed");
            }
            else
            {
                return Result.Nothing;
            }
            return Result.Success;
        }

        internal class CommandForm : Form, INotifyPropertyChanged
        {
            bool isEnabled = true;
            bool IsEnabled
            {
                get => isEnabled;
                set
                {
                    if (isEnabled != value)
                    {
                        isEnabled = value;
                        notifyPropertyChanged();
                    }
                }
            }
            internal CommandForm()
            {
                Title = "Command Form";
                Owner = RhinoEtoApp.MainWindow;
                DataContext = this;
                Width = 500;
                Resizable = false;
                this.BindDataContext(
                    f => f.Enabled,
                    (CommandForm vm) => vm.IsEnabled,
                    DualBindingMode.OneWay);

                var layout = new DynamicLayout();
                layout.Padding = 10;
                layout.Spacing = new Eto.Drawing.Size(5, 5);

                var btn = new Button();
                btn.Click += selectModels;
                btn.Text = "Select Models";

                var btn2 = new Button();
                btn2.Text = "OK";
                btn2.Click += (s, e) => Close();

                layout.AddRow(btn, null);
                layout.AddRow(btn2, null);
                Content = layout;

            }

            public event PropertyChangedEventHandler PropertyChanged;
            void notifyPropertyChanged([CallerMemberName] string pn = "")
            {
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(pn));
            }

            void selectModels(object sender, EventArgs e)
            {
                IsEnabled = false;
                Minimize();

                var go = new GetObject();
                go.SetCommandPrompt("Select the models");
                if (go.GetMultiple(1, 0) == GetResult.Object)
                {
                    foreach (var obj in go.Objects())
                    {
                        RhinoApp.WriteLine($"{obj.ObjectId}");
                    }
                }


                IsEnabled = true;
                BringToFront();

            }
        }

        internal class CommandDialog : Dialog, INotifyPropertyChanged
        {
            bool isEnabled = true;
            bool IsEnabled
            {
                get => isEnabled;
                set
                {
                    if (isEnabled != value)
                    {
                        isEnabled = value;
                        notifyPropertyChanged();
                    }
                }
            }
            internal CommandDialog()
            {
                Title = "Command Dialog";
                Owner = RhinoEtoApp.MainWindow;
                DataContext = this;
                Width = 500;
                this.BindDataContext(
                    f => f.Enabled,
                    (CommandDialog vm) => vm.IsEnabled,
                    DualBindingMode.OneWay);

                var layout = new DynamicLayout();
                layout.Padding = 10;
                layout.Spacing = new Eto.Drawing.Size(5, 5);

                var btn = new Button();
                btn.Click += selectModels;
                btn.Text = "Select Models";

                var btn2 = new Button();
                btn2.Text = "OK";
                btn2.Click += (s, e) => Close();

                layout.AddRow(btn, null);
                layout.AddRow(btn2, null);
                Content = layout;

            }

            public event PropertyChangedEventHandler PropertyChanged;
            void notifyPropertyChanged([CallerMemberName] string pn = "")
            {
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(pn));
            }

            void selectModels(object sender, EventArgs e)
            {
                IsEnabled = false;
                Minimize();

                var go = new GetObject();
                go.SetCommandPrompt("Select the models");
                if (go.GetMultiple(1, 0) == GetResult.Object)
                {
                    foreach (var obj in go.Objects())
                    {
                        RhinoApp.WriteLine($"{obj.ObjectId}");
                    }
                }

                IsEnabled = true;
                BringToFront();

            }
        }
    }
}
